/******************************************************************************/
/*                                                                            */
/*                Advanced Navigation Packet Protocol Library                 */
/*                Java Language GNSS Compass SDK, Version 7.0                 */
/*                    Copyright 2023, Advanced Navigation                     */
/*                                                                            */
/******************************************************************************/
/*
 * Copyright (C) 2023 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package com.advancednavigation.anPackets;

/* Sensor Ranges */
public class ANPacket184
{
	public static final int LENGTH = 4;

	public static final int ACCELEROMETER_RANGE_2G = 0;
	public static final int ACCELEROMETER_RANGE_4G = 1;
	public static final int ACCELEROMETER_RANGE_16G = 2;
	public static final int ACCELEROMETER_RANGE_MAXIMUM = 2;

	public static final int GYROSCOPES_RANGE_250 = 0;
	public static final int GYROSCOPES_RANGE_500 = 1;
	public static final int GYROSCOPES_RANGE_2000 = 2;
	public static final int GYROSCOPES_RANGE_MAXIMUM = 2;

	public static final int MAGNETOMETER_RANGE_2_GAUSS = 0;
	public static final int MAGNETOMETER_RANGE_4_GAUSS = 1;
	public static final int MAGNETOMETER_RANGE_8_GAUSS = 2;
	public static final int MAGNETOMETER_RANGE_MAXIMUM = 2;

	public Boolean permanent;
	public int accelerometersRange;
	public int gyroscopesRange;
	public int magnetometersRange;

	public ANPacket184()
	{
	}

	public ANPacket184(ANPacket packet)
	{
		permanent = packet.data[0] != 0;

		accelerometersRange = packet.data[1] & 0xFF;
		if (accelerometersRange < 0)
		{
			System.out.println("Packet184: Bad accelerometers range " + accelerometersRange);
			accelerometersRange = 0;
		}
		if (accelerometersRange > ACCELEROMETER_RANGE_MAXIMUM)
		{
			System.out.println("Packet184: Bad accelerometers range " + accelerometersRange);
			accelerometersRange = ACCELEROMETER_RANGE_MAXIMUM;
		}

		gyroscopesRange	= packet.data[2] & 0xFF;
		if (gyroscopesRange < 0)
		{
			System.out.println("Packet184: Bad gyroscopes range " + gyroscopesRange);
			gyroscopesRange = 0;
		}
		if (gyroscopesRange > GYROSCOPES_RANGE_MAXIMUM)
		{
			System.out.println("Packet184: Bad gyroscopes range " + gyroscopesRange);
			gyroscopesRange = GYROSCOPES_RANGE_MAXIMUM;
		}

		magnetometersRange = packet.data[3] & 0xFF;
		if (magnetometersRange < 0)
		{
			System.out.println("Packet184: Bad magnetometers range " + magnetometersRange);
			magnetometersRange = 0;
		}
		if (magnetometersRange > MAGNETOMETER_RANGE_MAXIMUM)
		{
			System.out.println("Packet184: Bad magnetometers range " + magnetometersRange);
			magnetometersRange = MAGNETOMETER_RANGE_MAXIMUM;
		}
	}

	public ANPacket toANPacket() throws IndexOutOfBoundsException
	{
		ANPacket packet = new ANPacket(LENGTH, ANPacket.PACKET_ID_SENSOR_RANGES);
		packet.data[0] = (byte) (permanent ? 1 : 0);
		if (accelerometersRange < 0 || accelerometersRange > ACCELEROMETER_RANGE_MAXIMUM) throw new IndexOutOfBoundsException();
		packet.data[1] = (byte) (accelerometersRange & 0xFF);
		if (gyroscopesRange < 0 || gyroscopesRange > GYROSCOPES_RANGE_MAXIMUM) throw new IndexOutOfBoundsException();
		packet.data[2] = (byte) (gyroscopesRange & 0xFF);
		if (magnetometersRange < 0 || magnetometersRange > MAGNETOMETER_RANGE_MAXIMUM) throw new IndexOutOfBoundsException();
		packet.data[3] = (byte) (magnetometersRange & 0xFF);
		return packet;
	}

	public String accelerometersRangeText()
	{
		switch(accelerometersRange)
		{
			case ACCELEROMETER_RANGE_4G:
				return "4 g";
			case ACCELEROMETER_RANGE_16G:
				return "16 g";
			case ACCELEROMETER_RANGE_2G:
			default:
				return "2 g";
		}
	}

	public String gyroscopesRangeText()
	{
		switch(gyroscopesRange)
		{
			case GYROSCOPES_RANGE_500:
				return "500 degrees/s";
			case GYROSCOPES_RANGE_2000:
				return "2000 degrees/s";
			case GYROSCOPES_RANGE_250:
			default:
				return "250 degrees/s";
		}
	}

	public String magnetometersRangeText()
	{
		switch(magnetometersRange)
		{
			case MAGNETOMETER_RANGE_4_GAUSS:
				return "4 Gauss";
			case MAGNETOMETER_RANGE_8_GAUSS:
				return "8 Gauss";
			case MAGNETOMETER_RANGE_2_GAUSS:
			default:
				return "2 Gauss";
		}
	}
}
